//
//  BFMusicManager.swift
//  BFRecordScreenKit
//
//  Created by Harry on 2022/3/4.
//

import Foundation
import BFMediaKit
import BFCommonKit
import BFNetRequestKit
import Alamofire


class BFMusicManager {
    
    var playStatusCallback : ((AVPlayerItem?, BFMuicInfoCellState) -> Void)?
    
    // 试听音乐
    let player:AVPlayer = {
        let p = AVPlayer(playerItem: nil)
        return p
    }()
    
    var loadedTimeRangesObserver : NSKeyValueObservation?
    var retaRangesObserver : NSKeyValueObservation?

    init() {
        loadedTimeRangesObserver = player.observe(\AVPlayer.currentItem?.loadedTimeRanges, options: [.new, .initial]) { [weak self] (player, change) in
            BFLog(1, message: "status: 0")

            DispatchQueue.main.async {[weak self] in
                guard let wself = self else { return }
                
                guard let ranges = change.newValue as? [CMTimeRange] else { return }
                if let totalDur = player.currentItem?.duration, totalDur.isValid,  CMTimeCompare(ranges.first?.duration ?? .zero, totalDur) >= 0{
                    BFLog(1, message: "status: 1")
                    wself.playStatusCallback?(wself.player.currentItem, .playing)
                }
            }
        }
        
    }
    
    func play() {
        player.play()
        
        DispatchQueue.main.asyncAfter(deadline: .now() + 0.1) { [weak self] in
            guard let wself = self else { return }
            
            if wself.player.rate == 1.0 {
                wself.playStatusCallback?(wself.player.currentItem, .playing)
            }
        }
    }
    
    func pause() {
        player.pause()
        playStatusCallback?(player.currentItem, .pause)
    }
    
    func stop() {
        playStatusCallback?(player.currentItem, .normal)

    }
    
    func replaceCurrentItem(newItemUrl: URL) {
        if player.currentItem != nil {
            NotificationCenter.default.removeObserver(self, name: .AVPlayerItemDidPlayToEndTime, object: player.currentItem)
        }
        player.replaceCurrentItem(with: AVPlayerItem(url: newItemUrl))
        NotificationCenter.default.addObserver(forName: .AVPlayerItemDidPlayToEndTime, object: player.currentItem, queue: .main) {[weak self] notice in
            guard let wself = self else { return }
            wself.player.currentItem?.seek(to: .zero, completionHandler: nil)
//                            wself.choseCell()?.status = .pause
//                            wself.chosedCellStatu = .pause
            wself.play()
        }
    }
    
    deinit {
        NotificationCenter.default.removeObserver(self)
        loadedTimeRangesObserver?.invalidate()
        loadedTimeRangesObserver = nil
    }

}


// class method
extension BFMusicManager {
    public class func downloadMusic(model: PQVoiceModel, complete:(@escaping () -> Void)) {

        if let path = model.musicPath, let url = URL(string: path) {
            
            let request = URLRequest(url: url)
            let session = URLSession.shared
            let downloadTask = session.downloadTask(with: request,
                   completionHandler: { (location:URL?, response:URLResponse?, error:Error?)
                    -> Void in
                print("location:\(location)")
                if let locationPath = location?.path {
                    BFMusicManager.moveLocation(locationPath: locationPath, model:model)
                }
                
                complete()
            })
            downloadTask.resume()
        }
        
        
    }
    
    fileprivate class func moveLocation(locationPath: String, model: PQVoiceModel) {
        let fileManager = FileManager.default
        let fileName = (model.musicPath! as NSString).lastPathComponent
        let documnets = bgMusicDirectory + fileName + ".mp3"
        createDirectory(path: bgMusicDirectory)
        do {
            try fileManager.moveItem(atPath: locationPath, toPath: documnets)
            model.wavFilePath = documnets
        } catch {
            BFLog(1, message: "移动失败, \(error)")
        }
        print("new location:\(documnets)")

    }
}
